<?php namespace EscapeWork\Manager\Providers;

use EscapeWork\Manager\Commands\ConfigureCommand;
use EscapeWork\Manager\Facades\UserAuth;
use EscapeWork\Manager\Models\RoleAction;
use EscapeWork\Manager\Models\User;
use EscapeWork\Manager\Permission;
use Illuminate\Support\ServiceProvider;
use Illuminate\Foundation\AliasLoader;

class ManagerServiceProvider extends ServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        $this->package('escapework/manager', 'manager', realpath(__DIR__ . '/../../..'));

        require_once(__DIR__ . '/../../../routes.php');
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->registerPermission();
        $this->registerCommands();
        $this->bindRepositories();
        $this->callProviders();
        $this->createAliases();
    }

    /**
     * Registering the permission check class
     */
    protected function registerPermission()
    {
        $this->app['escapework.manager.permission'] = $this->app->share(function($app)
        {
            $user = UserAuth::user();

            $permission = $app->make('EscapeWork\Manager\Permission', array($user, new RoleAction));
            $permission->fetchAllowedActions()
                       ->formatAllowedActionsArray();

            return $permission;
        });
    }

    /**
     * Calling the providers needed
     */
    protected function callProviders()
    {
        $this->app->register('EscapeWork\Manager\Providers\UserServiceProvider');

        if ($this->app['request']->is('manager/*')) {
            $this->app->register('EscapeWork\Manager\Providers\ErrorServiceProvider');
        }

        # escapework/msg
        $this->app->register('EscapeWork\Msg\Laravel\MsgServiceProvider');
    }

    /**
     * Binding the repositories
     */
    protected function bindRepositories()
    {
        $this->app->bind('EscapeWork\Manager\Repositories\UserRepository', 'EscapeWork\Manager\Models\User');
        $this->app->bind('EscapeWork\Manager\Repositories\UserReminderRepository', 'EscapeWork\Manager\Models\UserReminder');
        $this->app->bind('EscapeWork\Manager\Repositories\RoleRepository', 'EscapeWork\Manager\Models\Role');
        $this->app->bind('EscapeWork\Manager\Repositories\RoleActionRepository', 'EscapeWork\Manager\Models\RoleAction');
        $this->app->bind('EscapeWork\Manager\Repositories\UserRoleRepository', 'EscapeWork\Manager\Models\UserRole');
        $this->app->bind('EscapeWork\Manager\Repositories\PageRepository', 'EscapeWork\Manager\Models\Page');
        $this->app->bind('EscapeWork\Manager\Repositories\PageActionRepository', 'EscapeWork\Manager\Models\PageAction');

        $this->app->bind('EscapeWork\Manager\Permission', function($app, $parameters)
        {
            return new Permission($parameters[0], $parameters[1]);
        });
    }

    /**
     * Criando os alias
     */
    protected function createAliases()
    {
        $this->app->booting(function()
        {
            $loader = AliasLoader::getInstance();

            $loader->alias('Check', 'EscapeWork\Manager\Check');
            $loader->alias('Eloquent', 'EscapeWork\LaravelHelpers\Eloquent');
            $loader->alias('Msg', 'EscapeWork\Msg\Msg');
        });
    }

    /**
     * Registrando os comandos
     */
    public function registerCommands()
    {
        # limpando pedidos
        $this->app['command.manager.configure'] = $this->app->share(function($app)
        {
            return new ConfigureCommand(new User);
        });

        $this->commands('command.manager.configure');
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return array('escapework.manager.permission', 'command.manager.configure');
    }
}
